/* $Id: cm17a.c,v 1.15 2000/02/17 03:11:51 chrisf Exp $ */

/*
Sparkler - An X10 FireCracker (CM17A) tester for the PalmPilot
Copyright (C) 1999 Chris Faherty

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <Pilot.h>
#include <System/SerialMgr.h>
#include "callback.h"
#include "cm17a.h"
#include "m68328.h"

/* These are from SDK3.1 which I don't currently have installed */
#ifndef sysFtrNumProcessorID
#define sysFtrNumProcessorID 2
#endif
#ifndef sysFtrNumProcessorMask
#define sysFtrNumProcessorMask 0xFFFF0000
#endif
#ifndef sysFtrNumProcessor328
#define sysFtrNumProcessor328 0x00010000
#endif
#ifndef sysFtrNumProcessorEZ
#define sysFtrNumProcessorEZ 0x00020000
#endif

/* externs */
extern UInt SerialRefNum;

Word cm17a_cmd, cm17a_bits;
Boolean cm17a_iswait;
Word cm17a_state;
Word which_processor;

#define SET_BIT(x,b)    x|=b
#define CLEAR_BIT(x,b)  x&=~b
#define READ_BIT(x,b)   (x&b)
#define UART_RTS_LINE   0x02
#define UART_TXD_LINE   0x01
#define EZUART_RTS_LINE 0x40
#define EZUART_TXD_LINE 0x20

/*
 * Static function prototypes
 */

static Word Which_Processor(void);

/*
 * Called each time you want a bit to be sent to the CM17A that is connected
 * to the serial port.  The docs say the bit width should be at least .5ms
 * but most likely I am making it 10ms because that is the smallest tick.
 */
Boolean cm17a_tick(void)
{
    if (cm17a_iswait) {
        /* standby */
        cm17a_standby();
    } else {
        if (cm17a_bits >= 26) {
            /* still sending the header */
            if (CM17A_HDR & (1 << (cm17a_bits-26))) {
                /* on */
                cm17a_one();
            } else {
                /* off */
                cm17a_zero();
            }
        } else if ((cm17a_bits <= 9) && (cm17a_bits >= 2)) {
            /* the footer */
            if (CM17A_FTR & (1 << (cm17a_bits-2))) {
                /* on */
                cm17a_one();
            } else {
                /* off */
                cm17a_zero();
            }
        } else if (cm17a_bits >= 10) {
            /* the command */
            if (cm17a_cmd & (1 << (cm17a_bits-10))) {
                /* on */
                cm17a_one();
            } else {
                /* off */
                cm17a_zero();
            }
        }
        cm17a_bits--;
    }
    cm17a_iswait = !cm17a_iswait;
    return (Boolean) cm17a_bits;
}

/*
 * A logical "1" for the CM17A is:  RTS 1, DTR (TXD for us) 0.
 */
void cm17a_one(void)
{
    HwrDBallPtr pRegisters;

    pRegisters = (HwrDBallType *)hwr68328Base;

    if (!(cm17a_state & 2)) {
        switch (which_processor) {
        case 0: /* Dragonball */
            /* Make the RTS bit high */
            CLEAR_BIT(pRegisters->portMData, UART_RTS_LINE);
            break;
        case 1: /* Dragonball EZ */
            /* Make the RTS bit high */
            CLEAR_BIT(pRegisters->portEData, EZUART_RTS_LINE);
            break;
        }
    }

    if (cm17a_state & 1) {
        switch (which_processor) {
        case 0: /* Dragonball */
            /* Make the TXD bit low */
            SET_BIT(pRegisters->portMData, UART_TXD_LINE);
            break;
        case 1: /* Dragonball EZ */
            /* Make the TXD bit low */
            SET_BIT(pRegisters->portEData, EZUART_TXD_LINE);
            break;
        }
    }

    cm17a_state = 2;
}

/*
 * A logical "0" for the CM17A is:  RTS 0, DTR (TXD for us) 1.
 */
void cm17a_zero(void)
{
    HwrDBallPtr pRegisters;

    pRegisters = (HwrDBallType *)hwr68328Base;

    if (cm17a_state & 2) {
        switch (which_processor) {
        case 0: /* Dragonball */
            /* Make the RTS bit low */
            SET_BIT(pRegisters->portMData, UART_RTS_LINE);
            break;
        case 1: /* Dragonball EZ */
            /* Make the RTS bit low */
            SET_BIT(pRegisters->portEData, EZUART_RTS_LINE);
            break;
        }
    }

    if (!(cm17a_state & 1)) {
        switch (which_processor) {
        case 0: /* Dragonball */
            /* Make the TXD bit high */
            CLEAR_BIT(pRegisters->portMData, UART_TXD_LINE);
            break;
        case 1: /* Dragonball EZ */
            /* Make the TXD bit high */
            CLEAR_BIT(pRegisters->portEData, EZUART_TXD_LINE);
            break;
        }
    }
    
    cm17a_state = 1;
}

/*
 * Standby for the CM17A is:  RTS 1, DTR (TXD for us) 1.
 */
void cm17a_standby(void)
{
    HwrDBallPtr pRegisters;

    pRegisters = (HwrDBallType *)hwr68328Base;

    if (!(cm17a_state & 2)) {
        switch (which_processor) {
        case 0: /* Dragonball */
            /* Make the RTS bit high */
            CLEAR_BIT(pRegisters->portMData, UART_RTS_LINE);
            break;
        case 1: /* Dragonball EZ */
            /* Make the RTS bit high */
            CLEAR_BIT(pRegisters->portEData, EZUART_RTS_LINE);
            break;
        }
    }
    
    if (!(cm17a_state & 1)) {
        switch (which_processor) {
        case 0: /* Dragonball */
            /* Make the TXD bit high */
            CLEAR_BIT(pRegisters->portMData, UART_TXD_LINE);
            break;
        case 1: /* Dragonball EZ */
            /* Make the TXD bit high */
            CLEAR_BIT(pRegisters->portEData, EZUART_TXD_LINE);
            break;
        }
    }

    cm17a_state = 3;
}

/*
 * Reset for the CM17A is:  RTS 0, DTR (TXD for us) 0.
 */
void cm17a_reset(void)
{
    HwrDBallPtr pRegisters;

    pRegisters = (HwrDBallType *)hwr68328Base;

    which_processor = Which_Processor();

    switch (which_processor) {
    case 0: /* Dragonball */
        /* Now get control of RTS */
        SET_BIT(pRegisters->portMSelect, UART_RTS_LINE);
        /* Make RTS output ports */
        SET_BIT(pRegisters->portMDir, UART_RTS_LINE);
        /* Make the RTS bit low */
        SET_BIT(pRegisters->portMData, UART_RTS_LINE);
        /* Now get control of TXD */
        SET_BIT(pRegisters->portMSelect, UART_TXD_LINE);
        /* Make TXD output ports */
        SET_BIT(pRegisters->portMDir, UART_TXD_LINE);
        /* Make the TXD bit low */
        SET_BIT(pRegisters->portMData, UART_TXD_LINE);
        break;
    case 1: /* Dragonball EZ */
        /* Now get control of RTS */
        SET_BIT(pRegisters->portESelect, EZUART_RTS_LINE);
        /* Make RTS output ports */
        SET_BIT(pRegisters->portEDir, EZUART_RTS_LINE);
        /* Make the bit high */
        SET_BIT(pRegisters->portEData, EZUART_RTS_LINE);
        /* Now get control of TXD */
        SET_BIT(pRegisters->portESelect, EZUART_TXD_LINE);
        /* Make TXD output ports */
        SET_BIT(pRegisters->portEDir, EZUART_TXD_LINE);
        /* Make the TXD bit low */
        SET_BIT(pRegisters->portEData, EZUART_TXD_LINE);
        break;
    }

    cm17a_state = 0;
}

/*
 * Restore the UART back to normal
 */
void cm17a_restore(void)
{
    HwrDBallPtr pRegisters;

    pRegisters = (HwrDBallType *)hwr68328Base;

    switch (which_processor) {
    case 0: /* Dragonball */
        /* set RTS back to normal */
        CLEAR_BIT(pRegisters->portMSelect, UART_RTS_LINE);
        /* set TXD back to normal */
        CLEAR_BIT(pRegisters->portMSelect, UART_TXD_LINE);
        break;
    case 1: /* Dragonball EZ */
        /* set RTS back to normal */
        CLEAR_BIT(pRegisters->portESelect, EZUART_RTS_LINE);
        /* set TXD back to normal */
        CLEAR_BIT(pRegisters->portESelect, EZUART_TXD_LINE);
        break;
    }
}

static Word Which_Processor(void)
{
    DWord id, chip;
    Word which;

    /* Determine which processor */
    if (!FtrGet(sysFtrCreator, sysFtrNumProcessorID, &id)) {
        chip = id & sysFtrNumProcessorMask;
        if (chip==sysFtrNumProcessor328) which = 0;
        else if (chip==sysFtrNumProcessorEZ) which = 1;
        else which = 0;
    } else which = 0; /* feature is only in >= OS3.1 */

    return which;
}
