/* $Id: sparkler.c,v 1.26 2001/04/06 02:45:05 chrisf Exp $ */

/*
Sparkler - An X10 FireCracker (CM17A) tester for the PalmPilot
Copyright (C) 1999 Chris Faherty

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <Pilot.h>
#include <System/SerialMgr.h>
#include "callback.h"
#include "sparklerRsc.h"
#include "cm17a.h"

/* externs */

extern Word cm17a_cmd, cm17a_bits;
extern Boolean cm17a_iswait;
extern Word cm17a_state;

/* globals */

Err errno;
Word tps;
UInt SerialRefNum;
Boolean SerialOpen;

/*
 * Static function prototypes
 */
static Boolean MainFormHandleEvent(EventPtr e);
static Boolean ApplicationHandleEvent(EventPtr e);
static Word StartApplication(void);
static void StopApplication(void);
static void EventLoop(void);
static void *GetObjectPtr(const Int objectID);

/* locals */
static Int code;

static Boolean MainFormHandleEvent(EventPtr e)
{
    Boolean handled=false;
    FormPtr frm;
    
    CALLBACK_PROLOGUE

    switch (e->eType) {
    case frmOpenEvent:
        frm = FrmGetActiveForm();
        FrmDrawForm(frm);
        code = CM17A_A;
        CtlSetValue(GetObjectPtr(group1), true);
        handled = true;
        break;

    case menuEvent:
        MenuEraseStatus(NULL);
        switch (e->data.menu.itemID) {
        }
        handled = true;
        break;

    case popSelectEvent:
		if (e->data.popSelect.controlID == codeTrigger)
            {
			Int selection = e->data.popSelect.selection;
			if (selection != noListSelection)
                {
                switch(selection)
                    {
					case  0: code = CM17A_A; break;
					case  1: code = CM17A_B; break;
					case  2: code = CM17A_C; break;
					case  3: code = CM17A_D; break;
					case  4: code = CM17A_E; break;
					case  5: code = CM17A_F; break;
					case  6: code = CM17A_G; break;
					case  7: code = CM17A_H; break;
					case  8: code = CM17A_I; break;
					case  9: code = CM17A_J; break;
					case 10: code = CM17A_K; break;
					case 11: code = CM17A_L; break;
					case 12: code = CM17A_M; break;
					case 13: code = CM17A_N; break;
					case 14: code = CM17A_O; break;
					case 15: code = CM17A_P; break;
                    }
                CtlSetLabel(GetObjectPtr(codeTrigger),
                    LstGetSelectionText(e->data.popSelect.listP, selection));
                }
            }
        handled = true;
        break;

    case ctlSelectEvent:
        switch (e->data.ctlSelect.controlID) {
        case buttonID_1on:
            cm17a_cmd = code | CM17A_1 | CM17A_ON;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_1off:
            cm17a_cmd = code | CM17A_1 | CM17A_OFF;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_2on:
            cm17a_cmd = code | CM17A_2 | CM17A_ON;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_2off:
            cm17a_cmd = code | CM17A_2 | CM17A_OFF;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_3on:
            cm17a_cmd = code | CM17A_3 | CM17A_ON;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_3off:
            cm17a_cmd = code | CM17A_3 | CM17A_OFF;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_4on:
            cm17a_cmd = code | CM17A_4 | CM17A_ON;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_4off:
            cm17a_cmd = code | CM17A_4 | CM17A_OFF;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_5on:
            cm17a_cmd = code | CM17A_5 | CM17A_ON;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_5off:
            cm17a_cmd = code | CM17A_5 | CM17A_OFF;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_6on:
            cm17a_cmd = code | CM17A_6 | CM17A_ON;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_6off:
            cm17a_cmd = code | CM17A_6 | CM17A_OFF;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_7on:
            cm17a_cmd = code | CM17A_7 | CM17A_ON;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_7off:
            cm17a_cmd = code | CM17A_7 | CM17A_OFF;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_8on:
            cm17a_cmd = code | CM17A_8 | CM17A_ON;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_8off:
            cm17a_cmd = code | CM17A_8 | CM17A_OFF;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_dim:
            cm17a_cmd = code | CM17A_DIM;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        case buttonID_bright:
            cm17a_cmd = code | CM17A_BRIGHT;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
            break;
        }
        if(CtlGetValue(GetObjectPtr(group2)))
			cm17a_cmd |= 0x0400;

        break;

    default:
        break;
    }

    CALLBACK_EPILOGUE

    return handled;
}

static Boolean ApplicationHandleEvent(EventPtr e)
{
    FormPtr frm;
    Word formId;
    Boolean handled=false;

    if (e->eType == frmLoadEvent) {
        formId = e->data.frmLoad.formID;
        frm = FrmInitForm(formId);
        FrmSetActiveForm(frm);

        switch (formId) {
        case MainForm:
            FrmSetEventHandler(frm, MainFormHandleEvent);
            break;
        }
        handled = true;
    }

    return handled;
}

/* Get preferences, open (or create) app database */
static Word StartApplication(void)
{
    tps = SysTicksPerSecond();
    cm17a_bits = 0;
    SerialRefNum = 0;
    SerialOpen = false;
    /*
     * Load the serial library
     */
    errno = SysLibFind("Serial Library", &SerialRefNum);
    if (errno) {
        /*
         * BTW make sure there is " " (single space) in unused ^1 ^2 ^3
         * entries or PalmOS <= 2 will blow up.
         */
        FrmCustomAlert(ErrorAlert, "Failed To Load Library!", " ", " ");
        return 1;
    }

    FrmGotoForm(MainForm);
    return 0;
}

/* Save preferences, close forms, close app database */
static void StopApplication(void)
{
    if (SerialOpen) {
        cm17a_restore();
        errno = SerClose(SerialRefNum);
        if (errno) {
            FrmCustomAlert(ErrorAlert, "Error during Close!", " ", " ");
        }
        SerialOpen = false;
    }
    FrmSaveAllForms();
    FrmCloseAllForms();
}

/* The main event loop */
static void EventLoop(void)
{
    Word err;
    EventType e;
    long eventTimeout, ticks, tickticks, tickticks2, bitTimeout;

    tickticks = TimGetTicks();
    /* try to get 1/100th second timeouts */
/*    if ((bitTimeout = tps/100) == 0) bitTimeout = 1; */
    bitTimeout = 0; /* nah, just run it full blast */
    eventTimeout = evtWaitForever;
    do {
        EvtGetEvent(&e, eventTimeout);
        if (! SysHandleEvent (&e))
            if (! MenuHandleEvent (NULL, &e, &err))
                if (! ApplicationHandleEvent (&e))
                    FrmDispatchEvent (&e);

        /*
         * See if it is time to send another bit
         */
        ticks = TimGetTicks();
        if (cm17a_bits && ((ticks-tickticks) >= bitTimeout)) {
            tickticks = ticks;
            if (!SerialOpen) {
                /* need to open serial port */
                errno = SerOpen(SerialRefNum, 0, 9600);
                if (errno == serErrAlreadyOpen) {
                    FrmCustomAlert(ErrorAlert,
                        "Port already open!", " ", " ");
                    cm17a_bits = 0;
                } else if (errno) {
                    FrmCustomAlert(ErrorAlert,
                        "Failed to open port!", " ", " ");
                    cm17a_bits = 0;
                } else {
                    /* always use serial connector, no infrared */
                    /* SerControl(SerialRefNum, 10, NULL, NULL); */
                    SerialOpen = true;
                }
                if (SerialOpen) {
                    /*
                     * Send a reset signal and then go into standby for a
                     * little while prior to sending data.
                     */
                    cm17a_reset();
                    cm17a_standby();
                    eventTimeout = bitTimeout;
                    tickticks2 = ticks;
                }
            } else if (SerialOpen && ((ticks-tickticks2) >= 4)) {
                cm17a_tick();
            }
        }

        /*
         * Finish up the transmission with a delay in standby.
         */
        if (SerialOpen && !cm17a_bits && ((ticks-tickticks) >= (tps/8))) {
            tickticks = ticks;
            cm17a_restore();
            errno = SerClose(SerialRefNum);
            if (errno) {
                FrmCustomAlert(ErrorAlert,
                    "Error during Close!", " ", " ");
            }
            SerialOpen = false;
            eventTimeout = evtWaitForever;

/* this is for testing, keep sending the command */
/*
            eventTimeout = bitTimeout;
            cm17a_bits = CM17A_BITS;
            cm17a_iswait = true;
*/
        }

    } while (e.eType != appStopEvent);
}

static void *GetObjectPtr(const Int objectID)
{
	FormType *frm = FrmGetActiveForm();

	return FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, objectID));
}


/* Main entry point; it is unlikely you will need to change this except to
   handle other launch command codes */
DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
    Word err;

    if (cmd == sysAppLaunchCmdNormalLaunch) {

        err = StartApplication();
        if (err) return err;

        EventLoop();
        StopApplication();

    } else {
        return sysErrParamErr;
    }

    return 0;
}
